/**
 * 
 */
package ca.gerrymatte.webutils.mailserver;

import java.io.IOException;
import java.util.*;

import javax.naming.*;
import javax.mail.*;

import javax.servlet.ServletConfig;
import javax.servlet.GenericServlet;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import java.sql.*;
import javax.sql.*;
import javax.servlet.jsp.jstl.core.Config;

/**
 * @author Gerry Matte
 * This servlet runs at application startup and binds three application context attributes 
 * SQL_DATA_SOURCE - a JDBC DataSource to the Mail Server's database (standard default name used for JSTL pages)
 * MAIL_POSTOFFICES - an ArrayList populated with the names of all post offices sorted alphabetically
 * MAIL_SESSION - a JavaMail Session for sending eMail
 *
 */
public class DataSourceServlet extends GenericServlet {
	  /**
	 * Initial Version for a JSP datasource factory
	 */
	private static final long serialVersionUID = -827444607746006859L;
	  private static String INITIAL_CONTEXT_FACTORY="org.jnp.interfaces.NamingContextFactory";
	  private static String PROVIDER_URL="jnp://127.0.0.1:1099";
	  private static String JNDI_MAIL_DATASOURCE_NAME;
	  private static String JNDI_MAIL_SESSION_NAME;
	  static DataSource MAIL_DATASOURCE;
	  static Session MAIL_SESSION;
	  ArrayList postOffices=new ArrayList();

	/* (non-Javadoc)
	 * @see javax.servlet.GenericServlet#service(javax.servlet.ServletRequest, javax.servlet.ServletResponse)
	 */
	@Override
	public void service(ServletRequest arg0, ServletResponse arg1)
			throws ServletException, IOException {
		// TODO Auto-generated method stub

	}
	/** Initialise this servlet - normally done at startup **/
	public void init() throws ServletException {
		super.log("Initialising "+this.getServletName()+". ");
		boolean errors=false;
        
        JNDI_MAIL_DATASOURCE_NAME = this.getServletContext().getInitParameter("JNDI_MAIL_DATASOURCE_NAME");
        if(JNDI_MAIL_DATASOURCE_NAME==null) {
        	JNDI_MAIL_DATASOURCE_NAME = "Error: Missing context parameter JNDI_MAIL_DATASOURCE_NAME";
        	super.log(JNDI_MAIL_DATASOURCE_NAME);
        	errors=true;
        }
        JNDI_MAIL_SESSION_NAME = this.getServletContext().getInitParameter("JNDI_MAIL_SESSION_NAME");
        if(JNDI_MAIL_SESSION_NAME==null) {
        	JNDI_MAIL_SESSION_NAME = "Error: Missing context parameter JNDI_MAIL_SESSION_NAME";
        	super.log(JNDI_MAIL_SESSION_NAME);
        	errors=true;
        }
	      if(errors) throw new ServletException("Error obtaining context parameters");
	
	      // Initialise the JNDI context and obtain the container DataSource for the mail server database
	      Properties JndiProperties = new Properties();
	      JndiProperties.put(Context.INITIAL_CONTEXT_FACTORY, INITIAL_CONTEXT_FACTORY);
	      JndiProperties.put(Context.PROVIDER_URL, PROVIDER_URL);
	      InitialContext ctx;
	      try {
			ctx = new InitialContext(JndiProperties);
			MAIL_DATASOURCE =  (DataSource) ctx.lookup(JNDI_MAIL_DATASOURCE_NAME);
			// store the DataSource as an application attribute
			getServletContext().setAttribute("MAIL_DATASOURCE", MAIL_DATASOURCE);
			// and also as the default JSTL DataSource
			Config.set(getServletContext(), Config.SQL_DATA_SOURCE, MAIL_DATASOURCE);
			
	      	} catch (NamingException e) {
	      		super.log(e.getMessage());
	      		e.printStackTrace();
	      		errors=true;
	      		return;
	      	}
			// Now test a connection by loading the postoffice names to an ArrayList 
	      	// and then close the connection
	      	Connection conn;
	      	try {
				conn=MAIL_DATASOURCE.getConnection();
				Statement stmt = conn.createStatement();
				ResultSet rs = stmt.executeQuery("SELECT NAME AS PostOffice FROM postoffice ORDER BY NAME");
				int i=0;
				postOffices.clear();
			      if(rs!=null) {
			    	  while (rs.next()) {
			    		  postOffices.add(rs.getString("PostOffice"));
			    		  i++;
			    	  }
			    	  rs.close();
			    	  conn.close();
			      }
				super.log("A JDBC Connection was established from DataSource "+JNDI_MAIL_DATASOURCE_NAME+".");
				super.log(i+" post offices were found in the mail server database.");
				getServletContext().setAttribute("MAIL_POSTOFFICES", postOffices);
			} catch (SQLException e) {
				super.log("A JDBC Connection was NOT established from DataSource "+JNDI_MAIL_DATASOURCE_NAME+".");
				super.log(e.getMessage());
				e.printStackTrace();
			}
			// Now get the Mail Session and store it as an application scoped attribute named MAIL_SESSION
			try {
				MAIL_SESSION =  (Session) ctx.lookup(JNDI_MAIL_SESSION_NAME);
				super.log("A JavaMail Session was obtained from JNDI name "+JNDI_MAIL_SESSION_NAME+".");
				getServletContext().setAttribute("MAIL_SESSION", MAIL_SESSION);
			} catch (NamingException e) {
	      		super.log(e.getMessage());
	      		e.printStackTrace();
	      		errors=true;
	      		return;
			}

	}

}
